<?php

namespace App\Http\Controllers;

use App\Models\Blogs;
use App\Models\Category;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

class BlogsController extends Controller
{
    public function index()
    {
        if (request()->ajax()) {
            $blogs = Blogs::query();

            return DataTables::of($blogs)
                ->addColumn('category', function ($blog) {
                    return $blog->category->name;
                })
                ->addColumn('description', function ($blog) {
                    return $blog->description;
                })
                ->addColumn('image', function ($blog) {
                    return '<img src="' . asset($blog->image) . '" alt="Image" class="w-32">';
                })
                ->addColumn('action', function ($blog) {
                    return '<a href="' . route('blogs.edit', $blog->id) . '" class="text-black"><i class="text-4xl bx bx-edit"></i></a>
                            <form action="' . route('blogs.destroy', $blog->id) . '" method="POST" style="display: inline-block;">
                                ' . csrf_field() . '
                                ' . method_field('DELETE') . '
                                <button type="submit" class="text-red-500"><i class="text-4xl bx bx-trash"></i></button>
                            </form>
                           ';
                })
                ->rawColumns(['category', 'description', 'image', 'action'])
                ->make(true);
        }

        return view('admin.blogs.index'); // Ensure this view exists
    }

    public function create()
    {

        $category = Category::all();
        
        return view('admin.blogs.create', compact('category'));
    }

    public function store(Request $request)
    {
        // Validate the request
        $request->validate([
            'category_name' => 'required|string|max:255',
            'title' => 'required|string|max:255',
            'description' => 'nullable|string',
            'image' => 'required|image|mimes:jpg,jpeg,png,gif,webp|max:2048',
        ]);
    
        // Determine if the user selected an existing category or entered a new category
        $categoryName = $request->input('category_name');
        
        // Check if the category is the "Add New Category" option
        if ($categoryName === 'add_new') {
            $categoryName = $request->input('new_category'); // Get the new category input
            // Ensure that the new category input is provided
            if (!$categoryName) {
                return back()->withErrors(['category_name' => 'Please enter a new category name.']);
            }
        }
    
        // Find or create the category
        $category = Category::firstOrCreate(
            ['name' => $categoryName]
        );
    
        // Handle the image upload
        $image = null;
        if ($request->hasFile('image')) {
            $imageFile = $request->file('image');
            $imageName = time() . '_' . $imageFile->getClientOriginalName();
            $imageFile->move(public_path('images/uploads'), $imageName);
            // Set the image path if the upload is successful
            $image = 'images/uploads/' . $imageName;
        }
    
        // Create the blog entry with the associated category
        Blogs::create([
            'category_id' => $category->id,
            'title' => $request->input('title'),
            'description' => $request->input('description'),
            'image' => $image, // Store image path if available
        ]);
    
        return redirect()->route('blogs.index')->with('success', 'Blog created successfully!');
    }


    public function edit($id)
    {
        $blogs = Blogs::findOrFail($id);
        $categories  = Category::all();
        return view('admin.blogs.edit', compact('blogs','categories'));
    }

   public function update(Request $request, $id)
    {
        // Validate the request
        $request->validate([
            'category_name' => 'required|string|max:255',
            'title' => 'required|string|max:255',
            'description' => 'nullable|string',
            'image' => 'nullable|image|mimes:jpg,jpeg,png,gif,webp|max:2048',
        ]);
    
        $blog = Blogs::findOrFail($id);
    
        // Determine if the user selected an existing category or entered a new category
        $categoryName = $request->input('category_name');
        
        if ($categoryName === 'add_new') {
            $categoryName = $request->input('new_category');
            // Ensure that the new category input is provided
            if (!$categoryName) {
                return back()->withErrors(['category_name' => 'Please enter a new category name.']);
            }
        }
    
        // Find or create the category
        $category = Category::firstOrCreate(['name' => $categoryName]);
    
        // Handle the image upload if provided
        $image = $blog->image; // Default to existing image
        if ($request->hasFile('image')) {
            $imageFile = $request->file('image');
            $imageName = time() . '_' . $imageFile->getClientOriginalName();
            $imageFile->move(public_path('images/uploads'), $imageName);
            $image = 'images/uploads/' . $imageName;
        }
    
        // Update the blog with the new data
        $blog->update([
            'category_id' => $category->id,
            'title' => $request->input('title'),
            'description' => $request->input('description'),
            'image' => $image,
        ]);
    
        return redirect()->route('blogs.index')->with('success', 'Blog updated successfully!');
    }




    public function destroy($id)
    {
        $blogs = Blogs::findOrFail($id);
        $blogs->delete();

        return redirect()->back()->with('success', 'Blogs deleted successfully');
    }
}
