<?php

namespace App\Http\Controllers;

use App\Models\Brand;
use App\Models\Product;
use App\Models\Category;
use App\Models\ProductImage;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class ProductController extends Controller
{
    public function index()
    {
        if (request()->ajax()) {
            $product = Product::query();

            return DataTables::of($product)
                ->addColumn('brand', function ($products) {
                    return $products->brand->brand_name;
                })
                ->addColumn('category', function ($products) {
                    return $products->category->name;
                })
                ->addColumn('availability', function ($products) {
                    // Return 'In Stock' if availability is true, else return 'Out of Stock'
                    return $products->availability == 'In Stock' ? 'In Stock' : 'Out of Stock';
                })
                ->addColumn('featured', function ($products) {
                    // Return 'In Stock' if availability is true, else return 'Out of Stock'
                    return $products->featured == 'Featured' ? 'Featured' : 'Not Featured';
                })
                ->addColumn('action', function ($products) {
                    return '<a href="' . route('products.edit', $products->id) . '" class="text-black"><i class="text-4xl bx bx-edit"></i></a>
                            <form action="' . route('products.destroy', $products->id) . '" method="POST" style="display: inline-block;">
                                ' . csrf_field() . '
                                ' . method_field('DELETE') . '
                                <button type="submit" class="text-red-500"><i class="text-4xl bx bx-trash"></i></button>
                            </form>';
                })
                ->rawColumns(['category', 'action'])
                ->make(true);
        }

        return view('admin.products.index');
    }



    public function create()
    {

        $categories = Category::all();
        $brands = Brand::all();
        return view('admin.products.create', compact('categories', 'brands'));
    }



    public function store(Request $request)
    {
        // Validate the request
        $request->validate([
            'category_name' => 'required|string|max:255',
            'brand' => 'required|string|max:255',
            'name' => 'required|string|max:255',
            'price' => 'required|string|max:255',
            'stock' => 'required|string|max:255',
            'description' => 'nullable|string',
            'shipping_and_returns' => 'nullable|string',
            'image.*' => 'required|image|mimes:jpg,jpeg,png,gif,webp|max:2048', // Allow multiple images
        ]);

        // Handle category
        $categoryName = $request->input('category_name');
        if ($categoryName === 'add_new') {
            $categoryName = $request->input('new_category');
            // Ensure the new category input is provided
            if (!$categoryName) {
                return back()->withErrors(['category_name' => 'Please enter a new category name.']);
            }
        }
        $category = Category::firstOrCreate(['name' => $categoryName]);

        // Handle brand
        $brandName = $request->input('brand');
        if ($brandName === 'add_new') {
            $brandName = $request->input('new_brand');
            // Ensure the new brand input is provided
            if (!$brandName) {
                return back()->withErrors(['brand' => 'Please enter a new brand name.']);
            }
        }
        $brand = Brand::firstOrCreate(['brand_name' => $brandName]);

        // Create the product
        $product = Product::create([
            'category_id' => $category->id,
            'brand_id' => $brand->id,
            'name' => $request->input('name'),
            'price' => $request->input('price'),
            'availability' => "In Stock",
            'stock' => $request->input('stock'),
            'description' => $request->input('description'),
            'shipping_and_returns' => $request->input('shipping_and_returns'),
        ]);

        // Handle and save each uploaded image
        if ($request->hasFile('image')) {
            foreach ($request->file('image') as $image) {
                $imageName = time() . '_' . $image->getClientOriginalName();
                $image->move(public_path('images/uploads'), $imageName);

                // Save the image path to the product_images table
                ProductImage::create([
                    'product_id' => $product->id,
                    'image' => 'images/uploads/' . $imageName,
                ]);
            }
        }

        return redirect()->route('products.index')->with('success', 'Product created successfully with images!');
    }



    public function edit($id)
    {
        $product = Product::findOrFail($id);
        $categories = Category::all();
        $brands = Brand::all();
        return view('admin.products.edit', compact('product', 'categories', 'brands'));
    }

    public function update(Request $request, $id)
    {
        // Validate the request
        $request->validate([
            'category_name' => 'nullable|string|max:255',
            'brand' => 'nullable|string|max:255',
            'name' => 'nullable|string|max:255',
            'price' => 'nullable|string|max:255',
            'stock' => 'nullable|string|max:255',
            'description' => 'nullable|string',
            'shipping_and_returns' => 'nullable|string',
            'image.*' => 'nullable|max:2048', // Allow multiple images
        ]);

        // Find the existing product
        $product = Product::findOrFail($id);

        // Update or create category
        $categoryName = $request->input('category_name');
        if ($categoryName === 'add_new') {
            $category = Category::create(['name' => $request->input('new_category')]); // Create a new category
        } else {
            $category = Category::firstOrCreate(['name' => $categoryName]); // Use existing or create
        }

        // Update or create brand
        $brandName = $request->input('brand');
        if ($brandName === 'add_new') {
            $brand = Brand::create(['brand_name' => $request->input('new_brand')]); // Create a new brand
        } else {
            $brand = Brand::firstOrCreate(['brand_name' => $brandName]); // Use existing or create
        }

        // Update product details
        $product->update([
            'category_id' => $category->id,
            'brand_id' => $brand->id,
            'name' => $request->input('name'),
            'price' => $request->input('price'),
            'stock' => $request->input('stock'),
            'description' => $request->input('description'),
            'shipping_and_returns' => $request->input('shipping_and_returns'),
        ]);

        // Handle new images if uploaded
        if ($request->hasFile('image')) {
            foreach ($request->file('image') as $image) {
                $imageName = time() . '_' . $image->getClientOriginalName();
                $image->move(public_path('images/uploads'), $imageName);

                // Save each new image to the product_images table
                ProductImage::create([
                    'product_id' => $product->id,
                    'image' => 'images/uploads/' . $imageName,
                ]);
            }
        }

        // Optional: Delete specific images if requested (e.g., via checkboxes or hidden fields)
        if ($request->has('delete_image_ids')) {
            $deleteImageIds = $request->input('delete_image_ids');
            $currentImageCount = $product->images()->count(); // Assuming `images()` is the relationship method.

            foreach ($deleteImageIds as $imageId) {
                $image = ProductImage::find($imageId);

                // Proceed only if the image exists and belongs to the product
                if ($image && $image->product_id == $product->id) {
                    // Check if deleting this image would leave less than one image
                    if ($currentImageCount > 1) {
                        // Delete the image file from storage
                        unlink(public_path($image->image));
                        // Delete the image record from the database
                        $image->delete();
                        $currentImageCount--; // Decrease the count after deletion
                    } else {
                        // Optionally, you can log or notify that at least one image must remain
                        return back()->with('error', 'At least one image must remain.');
                    }
                }
            }
        }

        return redirect()->route('products.index')->with('success', 'Product updated successfully!');
    }


    public function updateAvailability($id)
    {
        // Retrieve the product by its ID
        $product = Product::findOrFail($id);

        // Get the availability status from the request (either "In Stock" or "Out of Stock")
        $availability = request('availability');

        // Update the product's availability field
        $product->availability = $availability;

        // Save the updated product data
        $product->save();

        // Respond with success message (optional)
        return response()->json(['success' => true]);
    }

    public function updateFeatured($id)
    {
        // Retrieve the product by its ID
        $product = Product::findOrFail($id);

        // Get the availability status from the request (either "In Stock" or "Out of Stock")
        $featured = request('featured');

        // Update the product's availability field
        $product->featured = $featured;

        // Save the updated product data
        $product->save();

        // Respond with success message (optional)
        return response()->json(['success' => true]);
    }

    public function destroy($id)
    {
        // Find the product by ID
        $product = Product::findOrFail($id);

        if ($product->orders()->exists()) {
            return redirect()->back()->with('errors', 'Cannot delete product with associated orders.');
        }

        foreach ($product->wishlist as $productwishlist) {
            $productwishlist->delete();
        }
        // Delete all related images from the product_images table
        $product->images()->delete();

        // Now delete the product itself
        $product->delete();

        return redirect()->back()->with('success', 'Product deleted successfully');
    }
}
