<?php

namespace App\Http\Controllers;

use App\Models\Product;
use App\Models\Subscription;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;
use App\Models\SubscriptionTransaction;
use Illuminate\Support\Facades\Session;
use Yajra\DataTables\Facades\DataTables;
use App\Models\SubscriptionTransactionDetail;

class SubscriptionController extends Controller
{
    public function index()
    {
        if (request()->ajax()) {
            $subscriptions = Subscription::query();

            return DataTables::of($subscriptions)
                ->editColumn('delivery_frequency', function ($subscription) {
                    switch ($subscription->delivery_frequency) {
                        case 7:
                            return 'Weekly';
                        case 14:
                            return 'Bi-weekly';
                        case 30:
                            return 'Monthly';
                        default:
                            return $subscription->delivery_frequency;
                    }
                })
                ->addColumn('upload_prescription', function ($subscription) {
                    return '<img src="' . asset($subscription->upload_prescription) . '" alt="" class="w-32">';
                })
                ->addColumn('discount_proof', function ($subscription) {
                    return '<img src="' . asset($subscription->discount_proof) . '" alt="" class="w-32">';
                })
                ->addColumn('action', function ($subscription) {
                    return '
                            <a href="' . route('edit.subscriptions', $subscription->id) . '" class="text-black"><i class="text-4xl bx bx-edit"></i></a>
                            <a href="' . route('subscriptions.show', $subscription->id) . '" class="text-black"><i class="text-4xl bx bx-clipboard"></i></a>
                            <form action="' . route('subscriptions.destroy', $subscription->id) . '" method="POST" style="display: inline-block;">
                                ' . csrf_field() . '
                                ' . method_field('DELETE') . '
                                <button type="submit" class="text-red-500"><i class="text-4xl bx bx-trash"></i></button>
                            </form>
                           ';
                })
                ->rawColumns(['upload_prescription', 'discount_proof', 'image', 'action'])
                ->make(true);
        }

        return view('admin.subscriptions.index'); // Ensure this view exists
    }
    public function getTransactionDetails($id)
    {
        $transaction = SubscriptionTransaction::with(['details.product'])->find($id);

        if (!$transaction) {
            return response()->json(['error' => 'Transaction not found'], 404);
        }

        $details = $transaction->details->map(function ($detail) {
            return [
                'product' => $detail->product,
                'quantity' => $detail->quantity,
                'subtotal' => $detail->subtotal,
            ];
        });

        return response()->json([
            'transaction' => $transaction,
            'details' => $details,
        ]);
    }

    public function show($id)
    {
        $subscription = Subscription::findOrFail($id);  // Retrieve the subscription model
        $subscription_id = $subscription->id;  // Get the subscription ID
        $payment_status = $subscription->payment_status;  // Get the subscription ID

        if (request()->ajax()) {
            $subscriptions = SubscriptionTransaction::where('subscription_id', $subscription_id)->with('subscription')->get();  // Fetch the subscription with related transactions

            return DataTables::of($subscriptions)
                ->addColumn('full_name', function ($subscription) {
                    return $subscription->subscription->full_name;
                })
                ->addColumn('total', function ($subscription) {
                    return '₱' . number_format($subscription->total, 2); // Add peso sign and format with 2 decimal places
                })
                ->addColumn('action', function ($subscription) {
                    return '
                    <a href="' . route('subscription.transaction.edit', $subscription->id) . '" class="text-black"><i class="text-4xl bx bx-edit"></i></a>
                    <a href="' . route('subscriptions.show_transaction_details', $subscription->id) . '" class="text-black"><i class="text-4xl bx bx-clipboard"></i></a>
                            <form action="' . route('subscriptions.destroy.transaction', $subscription->id) . '" method="POST" style="display: inline-block;">
                                ' . csrf_field() . '
                                ' . method_field('DELETE') . '
                                <button type="submit" class="text-red-500"><i class="text-4xl bx bx-trash"></i></button>
                            </form>
                           ';
                })
                ->rawColumns(['full_name', 'action'])
                ->make(true);
        }

        return view('admin.subscriptions.transactions', compact('subscription', 'subscription_id', 'payment_status'));
    }

    public function show_transaction_details($id)
    {
        $subscriptions = SubscriptionTransaction::findOrFail($id)->with('details')->first();

        return view('admin.subscriptions.show_transaction_details', compact('subscriptions'));
    }

    public function edit_subscription($id)
    {
        $subscription = Subscription::findOrFail($id);

        return view('admin.subscriptions.edit_subscription', compact('subscription'));
    }
    public function update_subscription(Request $request, $id)
    {
        $validatedData = $request->validate([
            'full_name' => 'nullable|string|max:255',
            'email' => 'nullable|email|max:255',
            'phone_number' => 'nullable|string|digits:11',
            'date_received' => 'nullable|date',
            'delivery_frequency' => 'nullable|string',
            'prescription' => 'nullable|string',
            'upload_prescription' => 'nullable|max:25600',
            'discount_proof' => 'nullable|max:25600',
        ]);

        $subscription = Subscription::findOrFail($id); // Fetch the existing record

        if ($request->hasFile('upload_prescription')) {
            // Remove the old file if it exists
            if ($subscription->upload_prescription && file_exists(public_path($subscription->upload_prescription))) {
                unlink(public_path($subscription->upload_prescription));
            }

            $file = $request->file('upload_prescription');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('prescriptions'), $fileName);
            $validatedData['upload_prescription'] = 'prescriptions/' . $fileName;
        }

        if ($request->hasFile('discount_proof')) {
            // Remove the old file if it exists
            if ($subscription->discount_proof && file_exists(public_path($subscription->discount_proof))) {
                unlink(public_path($subscription->discount_proof));
            }

            $file = $request->file('discount_proof');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('discount_proofs'), $fileName);
            $validatedData['discount_proof'] = 'discount_proofs/' . $fileName;
        }

        // Update the record
        $subscription->update($validatedData);

        return back()->with('success', 'Subscription updated successfully!');
    }

    public function edit($id)
    {
        $transaction = SubscriptionTransaction::findOrFail($id)->with('details')->first();
        $products = Product::all();

        return view('admin.subscriptions.edit_transaction_details', compact('transaction', 'products'));
    }

    public function update(Request $request, $id)
    {
        $validated = $request->validate([
            'subscription_id' => 'required|array',
            'product_id' => 'required|array',
            'price' => 'required|array',
            'quantity' => 'required|array',
            'note' => 'nullable|string',
        ]);

        $transaction = SubscriptionTransaction::findOrFail($id);

        // Update the transaction data
        $transaction->update([
            'note' => $validated['note'],
        ]);

        // Delete old transaction items and recreate them
        $transaction->details()->delete();

        $total = 0; // Initialize total amount
        foreach ($validated['product_id'] as $index => $productId) {
            $price = $validated['price'][$index];
            $quantity = $validated['quantity'][$index];
            $subtotal = $price * $quantity;

            $transaction->details()->create([
                'product_id' => $productId,
                'price' => $price,
                'quantity' => $quantity,
                'subtotal' => $subtotal, // Store subtotal in details
            ]);

            $total += $subtotal; // Accumulate total
        }

        // Update the total amount in the transaction
        $transaction->update([
            'total' => $total,
        ]);

        return redirect()->route('subscriptions.index')->with('success', 'Transaction updated successfully.');
    }


    public function create_transaction($id)
    {
        $subscription = Subscription::findOrFail($id);  // Retrieve the subscription model
        $subscription_id = $subscription->id;  // Get the subscription ID
        $products = Product::all();
        return view('admin.subscriptions.create_transaction', compact('subscription', 'subscription_id', 'products'));
    }


    public function store_transaction(Request $request)
    {
        // Validate the incoming request data
        $validated = $request->validate([
            'subscription_id' => 'required|array',
            'subscription_id.*' => 'exists:subscription,id',
            'product_id' => 'required|array',
            'product_id.*' => 'exists:product,id',
            'price' => 'required|array',
            'price.*' => 'numeric',
            'quantity' => 'required|array',
            'quantity.*' => 'integer|min:1',
            'note' => 'nullable|string',
        ]);

        // Save the main subscription transaction
        $transaction = new SubscriptionTransaction();
        $transaction->subscription_id = $request->subscription_id[0];
        $transaction->note = $request->note;
        $transaction->total = 0; // Will calculate the total later
        $transaction->payment_status = 'pending'; // Default status
        $transaction->save();

        $total = 0;

        // Save each transaction detail
        foreach ($request->product_id as $index => $productId) {
            $quantity = $request->quantity[$index];
            $price = $request->price[$index];
            $subtotal = $quantity * $price;
            $total += $subtotal;

            $detail = new SubscriptionTransactionDetail();
            $detail->subscription_transaction_id = $transaction->id;
            $detail->product_id = $productId;
            $detail->quantity = $quantity;
            $detail->subtotal = $subtotal;
            $detail->save();
        }

        // Update the total in the main transaction
        $transaction->total = $total;
        $transaction->save();

        // Redirect back with a success message
        return redirect()->route('subscriptions.show',   $transaction->subscription_id)->with('success', 'Transaction created successfully!');
    }

    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'full_name' => 'nullable|string|max:255',
            'email' => 'nullable|email|max:255',
            'phone_number' => 'nullable|string|digits:11',
            'date_received' => 'nullable|date',
            'delivery_frequency' => 'nullable|string',
            'prescription' => 'nullable|string',
            'upload_prescription' => 'nullable|file|mimes:jpg,jpeg,png,pdf|max:25600',
            'discount_proof' => 'nullable|file|mimes:jpg,jpeg,png,pdf|max:25600',
        ]);

        if ($request->hasFile('upload_prescription')) {
            $file = $request->file('upload_prescription');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('prescriptions'), $fileName);
            $validatedData['upload_prescription'] = 'prescriptions/' . $fileName;
        }

        if ($request->hasFile('discount_proof')) {
            $file = $request->file('discount_proof');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('discount_proofs'), $fileName);
            $validatedData['discount_proof'] = 'discount_proofs/' . $fileName;
        }

        $validatedData['user_id'] = Auth::id(); // Attach the user_id

        Subscription::create($validatedData);

        return back()->with('success', 'Subscription saved successfully!');
    }



    public function destroy($id)
    {
        $subscription = Subscription::findOrFail($id);

        // Delete related transactions first
        $subscription->transactions()->each(function ($transaction) {
            // Delete any related details or entries that may have foreign key constraints
            $transaction->details()->delete(); // Assuming there's a 'details' relationship on the transaction model
        });

        // Now delete the transaction records
        $subscription->transactions()->delete();

        // Finally, delete the subscription itself
        $subscription->delete();

        return redirect()->route('subscriptions.index')->with('success', 'Subscription deleted successfully');
    }

    public function destroy_transaction($id)
    {

        $transaction = SubscriptionTransaction::findOrFail($id);

        // Delete related rows in subscription_transaction_details
        $transaction->details()->delete();

        // Delete the transaction
        $transaction->delete();

        return redirect()->back()->with('success', 'Transaction deleted successfully');
    }


    public function processBillingAndPayment(Request $request)
    {
        $subscriptions = SubscriptionTransaction::findOrFail($request->order_id);

        // Ensure the selected payment method is passed in the request
        $selectedPaymentMethod = $request->input('payment_method'); // Assuming 'payment_method' is passed in the request

        // Validate the selected payment method
        if (!$selectedPaymentMethod) {
            return redirect()->back()->withErrors(['error' => 'No payment method selected.']);
        }

        // Optionally, validate that the selected payment method is one of the valid types (e.g., gcash, card)
        $validPaymentMethods = ['gcash', 'card'];
        if (!in_array($selectedPaymentMethod, $validPaymentMethods)) {
            return redirect()->back()->withErrors(['error' => 'Invalid payment method selected.']);
        }

        Session::put('subscription_transaction_id', $request->order_id);

        // Step 6: Prepare PayMongo API Request
        $response = Http::withHeaders([
            'Content-Type' => 'application/json',
            'accept' => 'application/json',
            'authorization' => 'Basic c2tfdGVzdF9YNHZ1elRQZ0Z6Q05SWWUxb3NaWnlnaGU6',
        ])->post('https://api.paymongo.com/v1/checkout_sessions', [
            'data' => [
                'attributes' => [
                    'line_items' => $this->buildLineItems($request),
                    'send_email_receipt' => true,
                    'show_description' => false,
                    'show_line_items' => true,
                    'cancel_url' => url('user_dashboard'),
                    'success_url' => route('subscription.billing.verify'),
                    'statement_descriptor' => 'Your Business Name',
                    'reference_number' => (string) $subscriptions->id,
                    'payment_method_types' => [$selectedPaymentMethod], // Use the selected payment method
                ],
            ],
        ]);

        $responseData = $response->json();

        if ($response->successful()) {
            // Update the subscription with the selected payment method
            $subscriptions->update([
                'transaction_id' => $responseData['data']['id'] ?? null,
                'payment_method' => $selectedPaymentMethod,  // Store the selected payment method
            ]);

            $checkoutUrl = $responseData['data']['attributes']['checkout_url'] ?? null;

            if ($checkoutUrl) {
                return redirect()->away($checkoutUrl);
            } else {
                return redirect()->back()->withErrors(['error' => 'Checkout URL not provided by PayMongo.']);
            }
        } else {
            return redirect()->back()->withErrors(['error' => $responseData['errors'][0]['detail'] ?? 'Payment processing failed.']);
        }
    }



    private function buildLineItems(Request $request)
    {
        $subscriptionTransaction = SubscriptionTransaction::with('details.product')->findOrFail($request->order_id);

        $lineItems = [];
        $totalFinalAmount = 0;

        // Loop through the transaction details
        foreach ($subscriptionTransaction->details as $detail) {
            $productName = $detail->product->name ?? 'Unknown Product';
            $quantity = (int) $detail->quantity; // Ensure quantity is an integer

            $productprice = $detail->product->price ?? 'Unknown Price';

            // Add the line item
            $lineItems[] = [
                'name' => $productName,
                'quantity' => $quantity,
                'amount' => (int) ($productprice * 100), // Convert to cents and ensure it's an integer
                'currency' => 'PHP',
            ];

            // Accumulate the final amount
            $totalFinalAmount += $productprice;
        }

        return $lineItems;
    }


    public function verify(Request $request)
    {
        $id = Session::get('subscription_transaction_id');

        $subscription = SubscriptionTransaction::findOrFail($id);

        $url = "https://api.paymongo.com/v1/checkout_sessions/{$subscription->transaction_id}";
        $response = Http::withHeaders([
            'Accept' => 'application/json',
            'Authorization' => 'Basic c2tfdGVzdF9YNHZ1elRQZ0Z6Q05SWWUxb3NaWnlnaGU6',
        ])->get($url);

        $status = $response['data']['attributes']['payments'][0]['attributes']['status'];

        if ($status === 'paid') {
            $subscription->update(['payment_status' => 'Paid']);
            Session::forget('subscription_transaction_id');
            return redirect()->route('user_dashboard')->with('success', 'Subscription Paid Successfully');
        } else {
            return redirect()->back()->withErrors(['error' => 'Payment failed.']);
        }
    }
}
